/**<pre>
 * Title: Easy GPS
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux, adapted from Richard Caudle
 * @version 1.0
 * </pre>
 */

package gps_monitor.gui;

import gps_monitor.gps.*;
import gps_monitor.util.*;
import transport.*;
import transport.protocol.*;


import java.awt.*;
import java.awt.event.*;
import java.util.Date;
import java.io.*;



public class GpsMonitorUI extends Frame implements ActionListener{


	int x,y;
	boolean track=false; /* true if we know the direction */
	boolean alwaysCenter=true;
	double heading = 180;

	Image background;

	ChartComp chart;
	ScrollPane pane;
	Label stat = new Label("");
	Button help;


	private Calling calling;

	private int port;


	//Constructs a UI which is used by 'widget' and displays info on 'user'
	public GpsMonitorUI(String user, boolean sim, int port){

		super(user);
		setSize(240,300);

		this.port = port;

		setLocation(0,0);
		setLayout(null);
		//parent = widget;

		this.addWindowListener(new WindowAdapter(){
	  		public void windowClosing(WindowEvent e)
			{
				System.exit(0);
			}
		});

		Toolkit tool = Toolkit.getDefaultToolkit();   //gotta love tool
		try{
			byte[] b = new byte[26000];
			FileInputStream fis = new FileInputStream("files/images/ipaq.jpg");
			fis.read(b);

			background = tool.createImage(b);
		}catch(Exception e){}


		Panel panel = new Panel(null);
		panel.setBounds( 0, 23, 240, 280);

    	pane=new ScrollPane();
    	chart = new ChartComp(this, true, 1);
    	pane.add(chart);
		help = new Button("Call for help");

		if(sim){
			setSize(370,603);
			panel.setBounds( 63, 161, 240, 280);
		}


		pane.setBounds( 8,16,224,218 );
		help.setBounds( 60,246, 120, 24);

		panel.add(pane);
		panel.add(help);

		add(panel);


		help.addActionListener(this);

	} //Constructor



	public void stopCall(){
		help.setLabel("Call for help");
		calling.stop();
		chart.stopCall();

	}


   	/* ====================================================================== */
	/** This
	 *
	 *  @param  g
	 *
	 */
   	public void paint(Graphics g) {

		// paint any space not covered
     	super.paint(g);

       	//int compWidth = getWidth();
     	//int compHeight = getHeight();
      	//int imageWidth, imageHeight;

      	// If we have a valid width and height for the
       	// background image, draw it.

         g.drawImage(background,0,23,this);



   	} //method: paintComponent



	public void actionPerformed(ActionEvent a){

		if( help.getLabel().equals("Call for help") ){
			help.setLabel("Cancel call for help");
			chart.startCall();
			calling = new Calling();
			calling.start();
		}else{
			help.setLabel("Call for help");
			calling.stop();
			chart.stopCall();
		}

	}


	/**
	 * This method is used to set the displayed latitude value on the GUI
	 *
	 * @param lat The latitude to be displayed
	 */
	public void setLatitude(String lat)
	{

	}

	/**
	 * This method is used to set the displayed longitude value on the GUI
	 *
	 * @param lon The longitude to be displayed
	 */
	public void setLongitude(String lon)
	{

	}

	/**
	 * This method is used to set the displayed altitude value on the GUI
	 *
	 * @param alt The altitude to be displayed
	 */
	public void setAltitude(Double alt)
	{

	}

	/**
	 * This method is used to set the displayed status on the GUI
	 *
	 * @param status The status to be displayed
	 */
	public void setStatus(String status)
	{

	}

	/**
	 * This method is used to set the displayed present accuracy value on the GUI
	 *
	 * @param accuracy The present accuracy to be displayed
	 */
	public void setPresentAccuracy(Double accuracy)
	{

	}

	/**
	 * This method is used to set the displayed mean accuracy value on the GUI
	 *
	 * @param accuracy The mean accuracy value to be displayed
	 */
	public void setMeanAccuracy(Double accuracy)
	{

	}

	/**
	 * This method is used to set the displayed speed value on the GUI
	 *
	 * @param speed The speed to be displayed
	 */
	public void setSpeed(Double speed)
	{

	}


	/**
	 * This method is for setting the current gps text line
	 *
	 * @param
	 */
	public void setStat(String s)
	{

	}


	/**
	 * This method is used to set the displayed heading value on the GUI
	 *
	 * @param heading The heading to be displayed
	 */
	public void setHeading(double heading)
	{

	}

	/**
	 * This method is used to set the displayed last update time value on the GUI
	 *
	 * @param time The last update time to be displayed
	 */
	public void setLastUpdate(Long time)
	{

	}

	/**
	 * This method is used to set the displayed OS reference on the GUI
	 *
	 * @param os The OS reference to be displayed
	 */
	public void setOSReference(String os)
	{

	}

	/**
	 * This method is used to set the displayed UTM reference on the GUI
	 *
	 * @param utm The UTM reference to be displayed
	 */
	public void setUTMReference(String utm)
	{

	}





	public ChartComp getChartComp(){
    	return(chart);
  	}




  	public void updatePos(Point pos, Point pred){



		chart.updatePos(pos,pred, heading);
		chart.paint();
		Point p=pane.getScrollPosition();
		Dimension d=pane.getViewportSize();
		Point p2=new Point(p);

		if ((track) && (!alwaysCenter)){
		  	if ((pos.y<this.y) && (pos.y<p.y+d.height/4)) {p2.y=pos.y-(2*d.height)/3;}
		  	if ((pos.x<this.x) && (pos.x<p.x+d.width/4)) {p2.x=pos.x-(2*d.width)/3;}
		  	if ((pos.y>this.y) && (pos.y>p.y+d.height-d.height/4)) {p2.y=pos.y-(d.height)/3;}
		  	if ((pos.x>this.x) && (pos.x>p.x+d.width-d.width/4)) {p2.x=pos.x-(d.width)/3;}

		}else{
		  	p2.y=pos.y-(d.height)/2;
		  	p2.x=pos.x-(d.width)/2;
		}

		if (!p2.equals(p)){
		  pane.setScrollPosition(p2);
		}

		this.x=pos.x;
		this.y=pos.y;
		track=true;
	}



	private class Calling extends Thread{

		public void run(){

			while(true){

				try{ Thread.sleep(2010); }catch(Exception e){}

				Message msg = new HelpMessage("help", port, 2, chart.getLocation() );

				SocketSender ss = new SocketSender(msg);
				ss.start();


			}

		}

	}



  	public void changeChart(String chartName, int scale, int precision){


    	if (!(new File("files/maps/"+chartName)).exists()){
      		System.err.println("Could not find file: "+chartName);
    	}

    	chart.loadChart(chartName, scale, precision);
    	this.show();
    	track=false;
  	}

  	public void timeOut(){
		chart.timeOut();
  	}

} //class: